function [lay] = prepare_layout(cfg, data);

% PREPARE_LAYOUT creates a 2-D layout of the channel locations. This layout
% is required for plotting the topographical distribution of the potential
% or field distribution, or for plotting timecourses in a topographical
% arrangement.
%
% Use as
%   lay = prepare_layout(cfg, data)
%
% There are several ways in which a 2-D layout can be made: it can be read
% directly from a *.lay file, it can be created based on 3-D electrode or
% gradiometer positions in the configuration or in the data, or it can be
% created based on the specification of an electrode of gradiometer file.
%
% You can specify either one of the following configuration options
%   cfg.layout
%   cfg.elec
%   cfg.grad
%   cfg.elecfile
%   cfg.gradfile
% Alternatively the layout can be constructed from either
%   data.grad
%   data.elec
%
% See also topoplotER, topoplotTFR, multiplotER, multiplotTFR

% Copyright (C) 2007, Robert Oostenveld
%
% $Log: prepare_layout.m,v $
% Revision 1.1  2007/03/14 08:44:29  roboos
% new function that replaces private/createlayout, this new function can be used by end-users
% added support for mat files containing a lay variable, made some changes to the lay structure
%

% set the defaults
if ~isfield(cfg, 'layout'),   cfg.layout = [];    end
if ~isfield(cfg, 'grad'),     cfg.grad = [];      end
if ~isfield(cfg, 'elec'),     cfg.elec = [];      end
if ~isfield(cfg, 'gradfile'), cfg.gradfile = [];  end
if ~isfield(cfg, 'elecfile'), cfg.elecfile = [];  end

if nargin<2
  data = [];
end

if isstr(cfg.layout) && filetype(cfg.layout, 'matlab')
  fprintf('reading layout from file %s\n', cfg.layout);
  load(cfg.layout, 'lay');

elseif isstr(cfg.layout) && filetype(cfg.layout, 'layout')
  fprintf('reading layout from file %s\n', cfg.layout);
  lay = readlay(cfg.layout);

elseif isstr(cfg.layout) && ~filetype(cfg.layout, 'layout')
  % assume that cfg.layout is an electrode file
  fprintf('creating layout from electrode file %s\n', cfg.layout);
  lay = elec2lay(read_fcdc_elec(cfg.layout));

elseif isstr(cfg.elecfile)
  fprintf('creating layout from electrode file %s\n', cfg.elecfile);
  lay = elec2lay(read_fcdc_elec(cfg.elecfile));

elseif ~isempty(cfg.elec) && isstruct(cfg.elec)
  fprintf('creating layout from cfg.elec\n');
  lay = elec2lay(cfg.elec);

elseif isfield(data, 'elec') && isstruct(data.elec)
  fprintf('creating layout from data.elec\n');
  lay = elec2lay(data.elec);

elseif isstr(cfg.gradfile)
  fprintf('creating layout from gradiometer file %s\n', cfg.gradfile);
  lay = grad2lay(read_fcdc_elec(cfg.gradfile));

elseif ~isempty(cfg.grad) && isstruct(cfg.grad)
  fprintf('creating layout from cfg.grad\n');
  lay = grad2lay(cfg.grad);

elseif isfield(data, 'grad') && isstruct(data.grad)
  fprintf('creating layout from data.grad\n');
  lay = grad2lay(data.grad);

else
  fprintf('reverting to 151 channel CTF default\n');
  lay = readlay('CTF151s.lay');
end

if ~any(strcmp('COMNT', lay.label))
  % add a placeholder for the comment in the upper left corner
  lay.label{end+1}  = 'COMNT';
  lay.width(end+1)  = mean(lay.width);
  lay.height(end+1) = mean(lay.height);
  X                 = min(lay.pos(:,1));
  Y                 = max(lay.pos(:,2));
  lay.pos(end+1,:)  = [X Y];
end

if ~any(strcmp('SCALE', lay.label))
  % add a placeholder for the scale in the upper right corner
  lay.label{end+1}  = 'SCALE';
  lay.width(end+1)  = mean(lay.width);
  lay.height(end+1) = mean(lay.height);
  X                 = max(lay.pos(:,1));
  Y                 = max(lay.pos(:,2));
  lay.pos(end+1,:)  = [X Y];
end

% to plot the layout for debugging, you can use this code snippet
global fb
if ~isempty(fb) && fb
  X      = lay.pos(:,1);
  Y      = lay.pos(:,2);
  Width  = lay.width;
  Height = lay.height;
  Lbl    = lay.label;
  figure
  plot(X, Y, '.');
  text(X, Y, Lbl);
  line([X X+Width X+Width X X]',[Y Y Y+Height Y+Height Y]');
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% SUBFUNCTION
% read the layout information from the ascii file
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function lay = readlay(filename);
if ~exist(filename, 'file')
  error(sprintf('could not open layout file: %s', filename));
end
[chNum,X,Y,Width,Height,Lbl,Rem] = textread(filename,'%f %f %f %f %f %q %q');
for i=1:length(Lbl)
  if ~isempty(Rem{i})
    % this ensures that channel names with a space in them are also supported (i.e. Neuromag)
    Lbl{i} = [Lbl{i} ' ' Rem{i}];
  end
end
lay.pos    = [X Y];
lay.width  = Width;
lay.height = Height;
lay.label  = Lbl;
return % function readlay

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% SUBFUNCTION
% convert 3D electrode positions into 2D layout
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function lay = elec2lay(elec);
prj = elproj(elec.pnt, 'stereographic'); % * [0 1; -1 0]
d = dist(prj');
d(find(eye(size(d)))) = inf;
mindist = min(d(:));
X = prj(:,1);
Y = prj(:,2);
Width  = ones(size(X)) * mindist * 0.8;
Height = ones(size(X)) * mindist * 0.6;
Lbl = elec.label;
lay.pos    = [X Y];
lay.width  = Width;
lay.height = Height;
lay.label  = Lbl;
return % function elec2lay

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% SUBFUNCTION
% convert the magnetometer/gradiometer coil positions into a layout
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function lay = grad2lay(grad);
fprintf('creating layout for %s system\n', sensortype(grad));
switch lower(sensortype(grad))
  case {'ctf151', 'ctf275'}
    Lbl = channelselection('MEG', grad.label);
    ind = match_str(grad.label, Lbl);
    pnt = grad.pnt(ind,:);
    prj = elproj(pnt, 'stereographic') * [0 1; -1 0];
    d = dist(prj');
    d(find(eye(size(d)))) = inf;
    mindist = min(d(:));
    X = prj(:,1);
    Y = prj(:,2);
    Width  = ones(size(X)) * mindist * 0.8;
    Height = ones(size(X)) * mindist * 0.6;

  case {'ctf151_planar', 'ctf275_planar'}
    % create a list with planar channel names
    chan = {};
    for i=1:length(grad.label)
      if ~isempty(findstr(grad.label{i}, '_dH')) || ...
          ~isempty(findstr(grad.label{i}, '_dV'))
        chan{i} = grad.label{i}(1:(end-3));
      end
    end
    chan = unique(chan);
    % find the matching channel-duplets
    ind = [];
    lab = {};
    for i=1:length(chan)
      ch1 =  [chan{i} '_dH'];
      ch2 =  [chan{i} '_dV'];
      sel = match_str(grad.label, {ch1, ch2});
      if length(sel)==2
        ind = [ind; i];
        lab(i,:) = {ch1, ch2};
        meanpnt1 = mean(grad.pnt(find(grad.tra(sel(1),:)),:), 1);
        meanpnt2 = mean(grad.pnt(find(grad.tra(sel(2),:)),:), 1);
        pnt(i,:) = mean([meanpnt1; meanpnt2], 1);
      end
    end
    lab = lab(ind,:);
    pnt = pnt(ind,:);
    prj = elproj(pnt, 'stereographic') * [0 1; -1 0];
    X = prj(:,1);
    Y = prj(:,2);
    d = dist(prj');
    d(find(eye(size(d)))) = inf;
    mindist = min(d(:));
    X1 = X; Y1 = Y + 0.21 * mindist;
    X2 = X; Y2 = Y - 0.21 * mindist;
    X = [X1; X2];
    Y = [Y1; Y2];
    Lbl = [lab(:,1); lab(:,2)];
    Width  = ones(size(X))*mindist/2;
    Height = ones(size(X))*mindist/3.0;

  case 'neuromag122'
    % find the matching channel-duplets
    ind = [];
    lab = {};
    for i=1:2:140
      ch1 = sprintf('MEG %03d', i);
      ch2 = sprintf('MEG %03d', i+1);
      sel = match_str(grad.label, {ch1, ch2});
      if length(sel)==2
        ind = [ind; i];
        lab(i,:) = {ch1, ch2};
        meanpnt1 = mean(grad.pnt(find(grad.tra(sel(1),:)),:), 1);
        meanpnt2 = mean(grad.pnt(find(grad.tra(sel(2),:)),:), 1);
        pnt(i,:) = mean([meanpnt1; meanpnt2], 1);
      end
    end
    lab = lab(ind,:);
    pnt = pnt(ind,:);
    prj = elproj(pnt, 'stereographic'); % * [0 1; -1 0];
    X = prj(:,1);
    Y = prj(:,2);
    d = dist(prj');
    d(find(eye(size(d)))) = inf;
    mindist = min(d(:));
    X1 = X; Y1 = Y + 0.21 * mindist;
    X2 = X; Y2 = Y - 0.21 * mindist;
    X = [X1; X2];
    Y = [Y1; Y2];
    Lbl = [lab(:,1); lab(:,2)];
    Width  = ones(size(X))*mindist/2;
    Height = ones(size(X))*mindist/3.0;

  case 'neuromag306'
    % find the matching channel-triplets
    ind = [];
    lab = {};
    for i=1:300
      ch1 = sprintf('MEG %03d1', i);
      ch2 = sprintf('MEG %03d2', i);
      ch3 = sprintf('MEG %03d3', i);
      sel = match_str(grad.label, {ch1, ch2, ch3});
      if length(sel)==3
        ind = [ind; i];
        lab(i,:) = {ch1, ch2, ch3};
        meanpnt1 = mean(grad.pnt(find(grad.tra(sel(1),:)),:), 1);
        meanpnt2 = mean(grad.pnt(find(grad.tra(sel(2),:)),:), 1);
        meanpnt3 = mean(grad.pnt(find(grad.tra(sel(3),:)),:), 1);
        pnt(i,:) = mean([meanpnt1; meanpnt2; meanpnt3], 1);
      end
    end
    lab = lab(ind,:);
    pnt = pnt(ind,:);
    prj = elproj(pnt, 'stereographic'); % * [0 1; -1 0];
    X = prj(:,1);
    Y = prj(:,2);
    d = dist(prj');
    d(find(eye(size(d)))) = inf;
    mindist = min(d(:));
    X1 = X - 0.2 * mindist; Y1 = Y + 0.2 * mindist;
    X2 = X - 0.2 * mindist; Y2 = Y - 0.2 * mindist;
    X3 = X + 0.2 * mindist; Y3 = Y + 0.0 * mindist;
    X = [X1; X2; X3];
    Y = [Y1; Y2; Y3];
    Lbl = [lab(:,1); lab(:,2); lab(:,3)];
    Width  = ones(size(X))*mindist/3;
    Height = ones(size(X))*mindist/3;

  case {'bti148', 'magnetometer' 'electrode'}
    % select all channels for the layout
    pnt = grad.pnt;
    lab = grad.label;
    prj = elproj(pnt, 'stereographic') * [0 1; -1 0];
    d = dist(prj');
    d(find(eye(size(d)))) = inf;
    mindist = min(d(:));
    X = prj(:,1);
    Y = prj(:,2);
    Lbl = lab;
    Width  = ones(size(X)) * mindist * 0.8;
    Height = ones(size(X)) * mindist * 0.6;

  otherwise
    error('unrecognized sensor type');
end % switch sensortype

lay.pos    = [X Y];
lay.width  = Width;
lay.height = Height;
lay.label  = Lbl;
return % function grad2lay
