function [stat] = timelockstatistics(cfg, varargin)

% TIMELOCKSTATISTICS  computes the probability for a given null-hypothesis using
% a parametric statistical test or using a non-parametric randomization test.
%
% Use as
%   [stat] = timelockstatistics(cfg, timelock1, timelock2, ...)
% where the input data is the result from either TIMELOCKANALYSIS or
% TIMELOCKGRANDAVERAGE.
%
% The configuration can contain the following options for data selection
%   cfg.channel     = Nx1 cell-array with selection of channels (default = 'all'),
%                     see CHANNELSELECTION for details
%   cfg.latency     = [begin end] in seconds or 'all' (default = 'all')
%   cfg.avgoverchan = 'yes' or 'no'                   (default = 'no')
%   cfg.avgovertime = 'yes' or 'no'                   (default = 'no')
%   cfg.parameter   = string                          (default = 'trial' or 'avg')
%
% Furthermore, the configuration should contain
%   cfg.method       = different methods for calculating the probability of the null-hypothesis,
%                    'montecarlo' use a non-parametric randomization test to get a Monte-Carlo estimate of the probability,
%                    'analytic'   use a parametric test that results in analytic probability,
%                    'stats'      use a parametric test from the Matlab statistics toolbox,
%                    'glm'        use a general linear model approach.
%
% The other cfg options depend on the method that you select. You
% should read the help of the respective subfunction STATISTICS_XXX
% for the corresponding configuration options and for a detailed
% explanation of each method.
%
% See also TIMELOCKANALYSIS, TIMELOCKGRANDAVERAGE

% This function depends on STATISTICS_WRAPPER

% Copyright (C) 2005-2006, Robert Oostenveld
%
% $Log: timelockstatistics.m,v $
% Revision 1.19  2006/11/27 15:38:20  roboos
% implemented support for cfg.parameter, by locally renaming the field in the data structure
%
% Revision 1.18  2006/10/19 15:05:51  roboos
% updated documentation
%
% Revision 1.17  2006/10/04 07:10:08  roboos
% updated documentation
%
% Revision 1.16  2006/07/12 09:18:17  roboos
% improved documentation
%
% Revision 1.15  2006/06/20 12:57:51  roboos
% updated documentation, removed support for Jens' old options
%
% Revision 1.14  2006/06/13 14:48:12  ingnie
% updated documentation

% the low-level data selection function does not know how to deal with other parameters, so work around it
if isfield(cfg, 'parameter')
  if strcmp(cfg.parameter, 'trial') || strcmp(cfg.parameter, 'individual')
     % this is dealt with correctly in the low-level code, even if an average is present
  elseif strcmp(cfg.parameter, 'avg')
    % this is only dealt with in the low-level code if no single-trial/individual data is present
    for i=1:length(varargin)
      if isfield(varargin{i}, 'trial')
        varargin{i} = rmfield(varargin{i}, 'trial');
      end
      if isfield(varargin{i}, 'individual')
        varargin{i} = rmfield(varargin{i}, 'individual');
      end
    end
  else
    % rename the parameter of interest into 'avg'
    fprintf('renaming parameter ''%s'' into ''avg''\n', cfg.parameter);
    for i=1:length(varargin)
      dat         = getsubfield(varargin{i}, cfg.parameter);
      varargin{i} = rmsubfield (varargin{i}, cfg.parameter);
      varargin{i} = setsubfield(varargin{i}, 'avg', dat);
      if isfield(varargin{i}, 'trial')
        varargin{i} = rmfield(varargin{i}, 'trial');
      end
      if isfield(varargin{i}, 'individual')
        varargin{i} = rmfield(varargin{i}, 'individual');
      end
    end
  end
end

% call the general function
[stat] = statistics_wrapper(cfg, varargin{:});

